@@ -1,6 +1,6 @@
 /*
  *  abc2ps: a program to typeset tunes written in abc format using PostScript
- *  Copyright (C) 1996,1997  Michael Methfessel
+ *  Copyright (C) 1996,1997,1998  Michael Methfessel
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *  The author can be contacted as follows:
  *  
  *  Michael Methfessel
- *  msm@ihp-ffo.de
+ *  methfessel@ihp-ffo.de
  *  Institute for Semiconductor Physics, PO Box 409, 
  *  D-15204 Frankfurt (Oder), Germany
  */
@@ -33,9 +33,9 @@
 
 /* -------------- general macros ------------- */
 
-#define VERSION              "1.2"      /* version */
-#define REVISION             "5"        /* revison */
-#define VDATE        "Oct 5 1997"       /* version date */
+#define VERSION              "1.3"      /* version */
+#define REVISION             "2"        /* revison */
+#define VDATE        "Aug 10 1998"      /* version date */
 #define VERBOSE0           2            /* default verbosity */
 #define DEBUG_LV           0            /* debug output level */
 #define OUTPUTFILE      "Out.ps"        /* standard output file */
@@ -81,24 +81,30 @@
 #define GSPACE0       10.0      /* space from grace note to big note */
 #define GSPACE         7.0      /* space between grace notes */
 #define DECO_IS_ROLL     0      /* ~ makes roll if 1, otherwise twiddle */
-#define CUT_NPLETS       0      /* 1 to always cut nplets off beams */
+#define WIDTH_MIN      1.0      /* minimal left,right width for xp list */
 #define RANFAC        0.05      /* max random shift = RANFAC * spacing */
+#define RANCUT        1.50      /* cutoff for random shift */
+#define BNUMHT        32.0      /* height for bar numbers */
 
 #define BETA_C          0.1     /* max expansion for flag -c */
 #define ALFA_X          1.0     /* max compression before complaining */
 #define BETA_X          1.2     /* max expansion before complaining */
 
+#define VOCPRE          0.4     /* portion of vocals word before note */
+#define GCHPRE          0.4     /* portion of guitar chord before note */
+
+#define DEFVOICE        "1"     /* default name for first voice */
+
 
 /* ----- macros for program internals ----- */
 
-
 #define CM             28.35    /* factor to transform cm to pt */
 #define PT              1.00    /* factor to transform pt to pt */
 #define IN             72.00    /* factor to transform inch to pt */
 
 #define STRL      301           /* string length in info fields */
 #define STRL1     101           /* string length for file names */
-#define MAXSYMS  2001           /* max symbols in tune */
+#define MAXSYMST   11           /* max symbols in start piece */
 #define MAXHD      10           /* max heads on one stem */
 #define NTEXT     100           /* max history lines for output */
 #define MAXINF    100           /* max number of input files */
@@ -134,6 +140,8 @@
 #define WORDS           14
 #define MWORDS          15
 #define PSCOMMENT       16
+#define TEMPO           17
+#define VOICE           18
 
 #define INVISIBLE    1          /* valid symbol types */
 #define NOTE         2         
@@ -164,6 +172,9 @@
 #define A_SH         1           /* codes for accidentals */
 #define A_NT         2
 #define A_FT         3
+#define A_DS         4
+#define A_DF         5
+
 
 #define D_GRACE      1           /* codes for decoration */
 #define D_STACC      2           
@@ -174,6 +185,8 @@
 #define D_DOWNBOW    7           
 #define D_ROLL       8           
 #define D_TRILL      9           
+#define D_HAT       10           
+#define D_ATT       11           
 
 #define H_FULL       1           /* types of heads */
 #define H_EMPTY      2
@@ -211,19 +224,25 @@
 
 
 #define MAXNTEXT   400            /* for text output */
-#define MAXWLEN     21
+#define MAXWLEN     51
 #define ALIGN        1
 #define RAGGED       2
 #define OBEYLINES    3
 #define OBEYCENTER   4
 #define SKIP         5
 
+#define E_CLOSED     1
+#define E_OPEN       2 
+
 
 
 /* ----- global variables ------- */
 
 int db=DEBUG_LV;           /* debug control */
 
+int maxSyms,maxVc;         /* for malloc */
+
+
 #define NCOMP     5        /* max number of composer lines */
 
 struct ISTRUCT {           /* information fields */
@@ -249,6 +268,7 @@ struct ISTRUCT {           /* information fields */
   char parts  [STRL];
   char xref   [STRL];
   char trans  [STRL];
+  char tempo  [STRL];
 } info,default_info;
 
 struct GRACE {             /* describes grace notes */
@@ -276,6 +296,7 @@ struct SYMBOL {            /* struct for a drawable symbol */
   float shac[MAXHD];           /* horizontal shift for accidentals */
   int npitch;                  /* number of note heads */
   int len;                     /* basic note length */
+  int fullmes;                 /* flag for full-measure rests */
   int word_st;                 /* 1 if word starts here */
   int word_end;                /* 1 if word ends here */
   int slur_st;                 /* how many slurs starts here */
@@ -293,24 +314,104 @@ struct SYMBOL {            /* struct for a drawable symbol */
   struct GRACE gr;             /* grace notes */
   struct DECO  dc;             /* grace notes */
   float xs,ys;                 /* position of stem end */
-  int u,v,w,t,z,q;             /* auxillary information */
+  int u,v,w,t,q;               /* auxillary information */
   int invis;                   /* mark note as invisible */
   float wl,wr;                 /* left,right min width */
   float pl,pr;                 /* left,right preferred width */
   float xl,xr;                 /* left,right expanded width */
-  float space,shrink,stretch;  /* glue before this symbol */
   int p_plet,q_plet,r_plet;    /* data for n-plets */
   float gchy;                  /* height of guitar chord */
   char text[41];               /* for guitar chords etc. */
   char *wordp[NWLINE];         /* pointers to wpool for vocals */
-} sym [MAXSYMS];
+  int p;                       /* pointer to entry in posit table */
+  float time;                  /* time for symbol start */
+} ;
+
+
+char lvoiceid[233];                   /* string from last V: line */
+int  nvoice,mvoice;                   /* number of voices defd, nonempty */
+int  ivc;                             /* current voice */
+int  ivc0;                            /* top nonempty voice */
+
+
+struct XPOS {            /* struct for a horizontal position */
+  int type;                    /* type of symbols here */
+  int next,prec;               /* pointers for linked list */ 
+  int eoln;                    /* flag for line break */
+  int *p;                      /* pointers to associated syms */
+  float time,dur;              /* start time, duration */
+  float wl,wr;                 /* right and left widths */
+  float space,shrink,stretch;  /* glue before this position */
+  float tfac;                  /* factor to tune spacings */
+  float x;                     /* final horizontal position */
+};
+
+
+int ixpfree;                   /* first free element in xp array */
+
+
+struct METERSTR {        /* data to specify the meter */
+  int meter1,meter2;
+  int mflag,lflag;
+  int dlen;
+  int insert;
+  char top[31];
+} default_meter;
+
+struct KEYSTR {          /* data to specify the key */
+  int ktype;
+  int sf;
+  int add_pitch;       
+  int root,root_acc;
+  int add_transp,add_acc[7];   
+} default_key;
+  
+
+struct VCESPEC {         /* struct to characterize a voice */
+  char id[33];                    /* identifier string, eg. a number */
+  char name[81];                  /* full name of this voice */
+  char sname[81];                 /* short name */
+  struct METERSTR meter,meter0,meter1;    /* meter */
+  struct KEYSTR   key,key0,key1;          /* keysig */
+  int stems;                      /* +1 or -1 to force stem direction */
+  int staves,brace,bracket;       /* for deco over several voices */
+  int do_gch;                     /* 1 to output gchords for this voice */
+  float sep;                      /* for space to next voice below */
+  int nsym;                       /* number of symbols */
+  int draw;                       /* flag if want to draw this voice */
+  int select;                     /* flag if selected for output */
+  int insert_btype,insert_num;    /* to split bars over linebreaks */
+  int insert_bnum;                /* same for bar number */
+  float insert_space;             /* space to insert after init syms */
+  int end_slur;                   /* for a-b slurs */
+  char insert_text[81];           /* string over inserted barline */
+};
+
+
+                              /* things to alloc: */
+struct SYMBOL  *sym;              /* symbol list */
+struct SYMBOL  **symv;            /* symbols for voices */
+struct XPOS    *xp;               /* shared horizontal positions */
+struct VCESPEC *voice;            /* characteristics of a voice */
+struct SYMBOL  **sym_st;          /* symbols a staff start */
+int            *nsym_st; 
+
+
+int halftones;                /* number of halftones to transpose by */
+
+                                          /* style parameters: */
+float f0p,f5p,f1p,f0x,f5x,f1x;            /*   mapping fct */
+float lnnp,bnnp,fnnp,lnnx,bnnx,fnnx;      /*   note-note spacing */
+float lbnp,bbnp,rbnp,lbnx,bbnx,rbnx;      /*   bar-note spacing */
+float lnbp,bnbp,rnbp,lnbx,bnbx,rnbx;      /*   note-bar spacing */
+
 
 char wpool[NWPOOL];            /* pool for vocal strings */
 int nwpool,nwline;             /* globals to handle wpool */
 
 struct SYMBOL zsym;            /* symbol containing zeros */
 
-struct BEAM {                  /* packages info on one beam */
+struct BEAM {                  /* packages info about one beam */
   int i1,i2;            
   float a,b;
   float x,y,t;
@@ -320,6 +421,7 @@ struct BEAM {                  /* packages info on one beam */
 struct FONTSPEC {
   char name [STRLFMT];
   float size;
+  int box;
 };
 
 struct FORMAT {                        /* struct for page layout */
@@ -327,13 +429,16 @@ struct FORMAT {                        /* struct for page layout */
   float    pageheight,staffwidth;
   float    topmargin,botmargin,leftmargin;
   float    topspace,wordsspace,titlespace,subtitlespace,partsspace;
-  float    composerspace,musicspace,staffsep,vocalspace,textspace;
-  float    scale,maxshrink,lineskipfac,parskipfac;
+  float    composerspace,musicspace,vocalspace,textspace;
+  float    scale,maxshrink,lineskipfac,parskipfac,indent;
+  float    staffsep,sysstaffsep,systemsep;
+  float    strict1,strict2;
   int      landscape,titleleft,continueall,breakall,writehistory;
   int      stretchstaff,stretchlast,withxrefs,barsperstaff;
-  int      oneperpage,musiconly,titlecaps;
-  struct FONTSPEC titlefont,subtitlefont,vocalfont,textfont;
-  struct FONTSPEC composerfont,partsfont,gchordfont,wordsfont;
+  int      oneperpage,titlecaps,barnums;
+  struct FONTSPEC titlefont,subtitlefont,vocalfont,textfont,tempofont;
+  struct FONTSPEC composerfont,partsfont,gchordfont,wordsfont,voicefont;
+  struct FONTSPEC barnumfont,barlabelfont;
 };
 
 struct FORMAT sfmt;                    /* format after initialization */
@@ -346,6 +451,7 @@ int  nfontnames;
 char txt[MAXNTEXT][MAXWLEN];           /* for output of text */
 int  ntxt; 
 
+char vcselstr[101];            /* string for voice selection */
 char mbf[501];                 /* mini-buffer for one line */
 char buf[BUFFSZ];              /* output buffer.. should hold one tune */
 int nbuf;                      /* number of bytes buffered */
@@ -370,16 +476,14 @@ int nsym;                      /* number of symbols in line */
 int nsym0;                     /* nsym at start of parsing a line */
 int pagenum;                   /* current page in output file */
 int xrefnum;                   /* xref number of current tune */
+int do_meter, do_indent;       /* how to start next block */
 
 char gch[201];                   /* guitar chord string */
-int meter1,meter2,dlen;          /* for meter, default length */
-int mflag,lflag;                 /* flags for C, l or s in meter */
 int bagpipe;                     /* switch for HP mode */
-char meter_top[81];              /* top part of meter, as string */
 int within_tune, within_block;   /* where we are in the file */
 int do_this_tune;                /* are we typesetting the current one ? */
 float posx,posy;                 /* overall scale, position on page */
-
+int barinit;                     /* carryover bar number between parts */
 
 char *p, *p0;                    /* global pointers for parsing music line */
 
@@ -387,22 +491,17 @@ int word,slur;                   /* variables used for parsing... */
 int last_note,last_real_note;
 int pplet,qplet,rplet;
 int carryover;                   /* for interpreting > and < chars */
-int end_slur_next_note;          /* for patched in a-b slurs */
 int ntinext,tinext[MAXHD];       /* for chord ties */
 
 struct {                         /* where to draw endings */
   float a,b;                        /* start and end position */
   int num;                          /* number of the ending */
+  int type;                         /* shape: open or closed at right */
 } ending[20];
 int num_ending;                  /* number of endings to draw */
 int mes1,mes2;                   /* to count measures in an ending */
 
-int nnote;                       /* count notes in one staff */
-int insert_btype,insert_num;     /* needed to split bars across lines */
 int slur1[20],slur2[20];         /* needed for drawing slurs */
-int keytype,transpose,sharps_flats,y_add;       
-int insert_meter;                /* flag to insert time signature */
-int meter_flag;                  /* similar... */
 int overfull;                    /* flag if staff overfull */
 int do_words;                    /* flag if staff has words under it */
 
@@ -426,12 +525,14 @@ int pretty;                        /* for pretty but sprawling layout */
 int bars_per_line;                 /* bars for auto linebreaking */
 int continue_lines;                /* flag to continue all lines */
 int landscape;                     /* flag for landscape output */
-int music_only;                    /* no vocals if 1 */
+int barnums;                       /* interval for bar numbers */
 float alfa_c;                      /* max compression allowed */
 float scalefac;                    /* scale factor for symbol size */
 float lmargin;                     /* left margin */
 float swidth;                      /* staff width */
-float staffsep;                    /* staff separation */
+float staffsep,dstaffsep;          /* staff separation */
+float strict1,strict2;             /* 1stave, mstave strictness */
+char transpose[21];                /* target key for transposition */
 
 float alfa_last,beta_last;         /* for last short short line.. */
 
@@ -456,9 +557,13 @@ int  psel[MAXINF];               /* pointers from files to selectors */
 
 int temp_switch;
 
+
 #include "style.h"               /* globals to define layout style */
 
 /* ----- include subroutine files ----- */
+void identify_note (struct SYMBOL *s, char *q);
+int parse_length ();
+
 #include "syms.h" 
 #include "util.h" 
 #include "pssubs.h" 
@@ -480,11 +585,17 @@ char *argv[];
   int isel,j,npat,search_field,urgc,retcode,rc1;
   
   /* ----- set default options and parse arguments ----- */
+
+  maxSyms = 800;
+  maxVc   = 4;
+
   init_ops (1);
   retcode=parse_args (argc, argv);
   if (retcode) exit (1);
   if (interactive || (do_mode==DO_OUTPUT)) 
-    printf ("This is abc2ps, version %s (%s)\n", VERSION, VDATE); 
+    printf ("This is abc2ps, version %s.%s (%s)\n", VERSION, REVISION, VDATE); 
+
+  alloc_structs ();
 
   /* ----- set the page format ----- */
   nfontnames=0;
@@ -510,7 +621,6 @@ char *argv[];
   /* ----- initialize ----- */
   zero_sym();
   pagenum=0;
-  transpose=0;
   tunenum=tnum1=tnum2=0;
   verbose=0;
   file_open=file_initialized=0;
@@ -528,7 +638,9 @@ char *argv[];
 
     if (interactive) {
       printf ("\nSelect tunes: ");
-      gets (aaa);
+/*|   gets (aaa); |*/
+/*|   fgets (aaa, sizeof(aaa), stdin); |*/
+      getline(aaa,500,stdin);
       if (isblank(aaa)) break;
       sscanf(aaa,"%s",ccc);
       if (ccc[0]=='?') {
