@@ -1,5 +1,6 @@
 /*  
- *  This file is part of abc2ps, Copyright (C) 1996,1997 Michael Methfessel
+ *  This file is part of abc2ps, 
+ *  Copyright (C) 1996,1997,1998  Michael Methfessel
  *  See file abc2ps.c for details.
  */
 
@@ -8,6 +9,10 @@
 /* ----- write_help ----- */
 void write_help ()
 {
+
+  printf ("abc2ps v%s.%s (%s, %s) compiled %s\n", 
+          VERSION, REVISION, VDATE, STYLE, __DATE__);
+
   printf ("Usage: abc2ps files..  [-e nums-or-pats] [other flags]\n"
           "  - show index of abc files or typeset tunes in Postscript.\n"
           "where: files   input files in abc format\n"
@@ -20,7 +25,6 @@ void write_help ()
           "       -O =    make outfile name from infile/title\n" 
           "       -i      run in interactive mode\n"
           "       -v nn   set verbosity level to nn\n"
-          "       -V      show program version\n"
           "       -h      show this command summary\n"
           "  Selection:\n"
           "       -e      following arguments are selectors\n"
@@ -29,7 +33,7 @@ void write_help ()
           "       -C      search Composer field instead of title\n"
           "       -R      search  Rhythm  field instead of title\n"
           "       -S      search  Source  field instead of title\n"
-          "       -A      select all tunes (override other selectors)\n"
+          "       -V str  select voices, eg. -V 1,4-5\n"
           "  Formatting:\n"
           "       -H      show the format parameters\n"
           "       -p      pretty output (looks better, needs more space)\n"
@@ -39,6 +43,7 @@ void write_help ()
           "       -m xx   set left margin (cm/in/pt)\n"
           "       -d xx   set staff separation (cm/in/pt)\n"
           "       -x      include xref numbers in output\n"
+          "       -k nn   number every nn bars; 0 for first in staff\n"
           "       -n      include notes and history in output\n"
           "       -N      write page numbers\n"
           "       -1      write one tune per page\n"
@@ -46,12 +51,19 @@ void write_help ()
           "       -g xx   set glue mode to shrink|space|stretch|fill\n"
           "       -F foo  read format from \"foo.fmt\"\n"
           "       -D bar  look for format files in directory \"bar\"\n"
+          "       -X x    set strictness for note spacing, 0<x<1 \n"
+          "  Transposing:\n"
+          "       -t n    transpose by n halftones (_n for negative number)\n"
+          "       -t XX   transpose root up/down to note XX (eg. C A# ^Bb _Bb)\n"
           "  Line breaks:\n"
           "       -a xx   set max shrinkage to xx (between 0 and 1)\n"
           "       -b      break at all line ends (ignore continuations)\n"
           "       -c      continue all line ends (append '\\')\n"
           "       -B bb   put line break every bb bars\n"
-          );
+          "  Alloc options:\n"
+          "     -maxs n   set maximal number of symbols (default %d)\n"
+          "     -maxv n   set maximal number of voices (default %d)\n"
+          , maxSyms, maxVc);
 }
 
 
@@ -121,16 +133,21 @@ int job;
   swidth               = -1.0;
   write_history        = -1;
   staffsep             = -1;
+  dstaffsep            =  0;
   break_continues      = -1;
   continue_lines       = -1;
   include_xrefs        = -1;
-  music_only           = -1;
   alfa_c               = -1.0;
+  strict1              = -1.0;
+  strict2              = -1.0;
+  barnums              = -1;
 
   select_all           = 0;
   do_mode              = DO_INDEX;
   pagenumbers          = 0;
   strcpy (styf, "");
+  strcpy (transpose, "");
+  strcpy (vcselstr, "");
 
   if (job) {
     strcpy (styd, DEFAULT_FDIR);
@@ -157,12 +174,17 @@ struct FORMAT *fmt;
   if (continue_lines >= 0)   fmt->continueall=continue_lines;
   if (break_continues >= 0)  fmt->breakall=break_continues;
   if (write_history >= 0)    fmt->writehistory=write_history;
-  if (bars_per_line >= 0)    fmt->barsperstaff=bars_per_line;
+  if (bars_per_line > 0)     fmt->barsperstaff=bars_per_line;
   if (include_xrefs >= 0)    fmt->withxrefs=include_xrefs;
-  if (staffsep >= 0)         fmt->staffsep=staffsep;
   if (one_per_page >= 0)     fmt->oneperpage=one_per_page;
-  if (music_only >= 0)       fmt->musiconly=music_only;
   if (alfa_c >= 0)           fmt->maxshrink=alfa_c;
+  if (staffsep >= 0)         fmt->staffsep=staffsep;
+  if (strict1 >= 0)          fmt->strict1=strict1;
+  if (strict2 >= 0)          fmt->strict2=strict2;
+  if (barnums >= 0)          fmt->barnums=barnums;
+  fmt->staffsep    = fmt->staffsep    + dstaffsep;
+  fmt->sysstaffsep = fmt->sysstaffsep + dstaffsep;
+
 }
 
 
@@ -194,11 +216,9 @@ char *av[];
         else if (c=='c')  continue_lines=0;
         else if (c=='x')  include_xrefs=0;
         else if (c=='1')  one_per_page=0;
-        else if (c=='A')  select_all=0;
         else if (c=='B')  bars_per_line=0;
         else if (c=='i')  interactive=0;
         else if (c=='n')  write_history=0;
-        else if (c=='M')  music_only=0;
         else if (c=='l')  landscape=0;
         else if (c=='p')  pretty=0;
         else if (c=='E')  epsf=0;
@@ -206,141 +226,188 @@ char *av[];
         else if (c=='N')  pagenumbers=0;
         else if (c=='O')  { choose_outname=0; strcpy (outf, OUTPUTFILE); }
         else {
-          printf ("++++ Cannot switch off flag: +%c\n", c);
+          printf ("+++ Cannot switch off flag: +%c\n", c);
           return 1; 
         }
         m++;
       }
     }
     
-    else if (av[i][0]=='-') {        /* interpret a flag with '-'*/
-      m=1;
-      k=strlen(av[i]);
-      while (m<k) {
-        c=av[i][m];
-        if      (c=='h')  help_me=1;                 /* simple flags */
-        else if (c=='H')  help_me=2;     
-        else if (c=='e')  sel_arg=1;
-        else if (c=='f')  {
-          nsel++;
-          strcpy (sel_str[nsel], "");
-          if (ninf==0) {                /* selector before first file */
-            strcpy(sel_str[nsel],sel_str[nsel-1]);
-            s_field[nsel]=s_field[nsel-1];
+    else if (av[i][0]=='-') {         /* interpret a flag with '-'*/
+      
+                                      /* identify fullword options first */
+      if (!strcmp(av[i],"-maxv")) {
+        if ((i==ac-1)||(av[i+1][0]=='-')) rx("missing parameter for ",av[i]);
+        sscanf(av[++i],"%d",&maxVc);
+      }
+      else if (!strcmp(av[i],"-maxs")) {
+        if ((i==ac-1)||(av[i+1][0]=='-')) rx("missing parameter for ",av[i]);
+        sscanf(av[++i],"%d",&maxSyms);
+      }
+      
+      else {
+        m=1;
+        k=strlen(av[i]);
+        while (m<k) {
+          c=av[i][m];
+          if      (c=='h')  help_me=1;                 /* simple flags */
+          else if (c=='H')  help_me=2;     
+          else if (c=='e')  sel_arg=1;
+          else if (c=='f')  {
+            nsel++;
+            strcpy (sel_str[nsel], "");
+            if (ninf==0) {                /* selector before first file */
+              strcpy(sel_str[nsel],sel_str[nsel-1]);
+              s_field[nsel]=s_field[nsel-1];
+            }
+            s_field[nsel]=search_field0;
+            sel_arg=0;
+            f_pos=i;
           }
-          s_field[nsel]=search_field0;
-          sel_arg=0;
-          f_pos=i;
-        }
-        else if (c=='b')  {break_continues=1; continue_lines=0;}
-        else if (c=='c')  {continue_lines=1; break_continues=0;}
-        else if (c=='x')  include_xrefs=1;
-        else if (c=='1')  one_per_page=1;
-        else if (c=='A')  select_all=1;
-        else if (c=='i')  interactive=1;
-        else if (c=='n')  write_history=1;
-        else if (c=='l')  landscape=1;
-        else if (c=='M')  music_only=1;
-        else if (c=='p')  pretty=1;
-        else if (c=='P')  pretty=2;
-        else if (c=='E')  epsf=1;
-        else if (c=='o')  do_mode=DO_OUTPUT;
-        else if (c=='N')  pagenumbers=1;
-        else if (strchr("TCRS",c)) {           
-          if (c=='T') s_field[nsel]=S_TITLE; 
-          if (c=='C') s_field[nsel]=S_COMPOSER;
-          if (c=='R') s_field[nsel]=S_RHYTHM; 
-          if (c=='S') s_field[nsel]=S_SOURCE; 
-          sel_arg=1; 
-        }
-        else if (c=='V')  {
-          write_version ();
-          if (!interactive) exit (0);
-          return 1;
-        }
-
-        else if (strchr("vsdwmgDFYBaO",c)) {      /* flags with parameter.. */
-          strcpy (aaa, &av[i][m+1]);
-          if ((strlen(aaa)>0) && strchr("glO",c)) {     /* no sticky arg */
-            printf ("++++ Incorrect usage of flag -%c\n", c);
-            return 1; 
+          else if (c=='b')  {break_continues=1; continue_lines=0;}
+          else if (c=='c')  {continue_lines=1; break_continues=0;}
+          else if (c=='x')  include_xrefs=1;
+          else if (c=='1')  one_per_page=1;
+          else if (c=='i')  interactive=1;
+          else if (c=='n')  write_history=1;
+          else if (c=='l')  landscape=1;
+          else if (c=='p')  pretty=1;
+          else if (c=='P')  pretty=2;
+          else if (c=='E')  epsf=1;
+          else if (c=='o')  do_mode=DO_OUTPUT;
+          else if (c=='N')  pagenumbers=1;
+          else if (strchr("TCRS",c)) {           
+            if (c=='T') s_field[nsel]=S_TITLE; 
+            if (c=='C') s_field[nsel]=S_COMPOSER;
+            if (c=='R') s_field[nsel]=S_RHYTHM; 
+            if (c=='S') s_field[nsel]=S_SOURCE; 
+            sel_arg=1; 
           }
           
-          got_value=1;                              /* check for value */
-          if (strlen(aaa)==0) {
-            i++; 
-            if ((i>=ac) || (av[i][0]=='-')) 
+          else if (strchr("vsdwmgtkDFYBVXaO",c)) {  /* flags with parameter.. */
+            strcpy (aaa, &av[i][m+1]);
+            if ((strlen(aaa)>0) && strchr("glO",c)) {     /* no sticky arg */
+              printf ("+++ Incorrect usage of flag -%c\n", c);
+              return 1; 
+            }
+            
+            got_value=1;                              /* check for value */
+            if (strlen(aaa)==0) {
               got_value=0;
-            else
-              strcpy (aaa,av[i]);
-          }
-          
-          if (got_value && strchr("vsYB",c)) {      /* check num args */
-            ok=1;
-            for (j=0;j<strlen(aaa);j++)
-              if (!strchr("0123456789.",aaa[j])) ok=0;
-            if (!ok) {
-              printf ("++++ Invalid parameter <%s> for flag -%c\n",aaa,c);
-              return 1;
+              i++; 
+              if ((i>=ac) || (av[i][0]=='-')) 
+                i--;
+              else {
+                strcpy (aaa,av[i]);
+                got_value=1;
+              }
             }
-          }
-          
-          if (!got_value) {                   /* check value was given */
-            printf ("++++ Missing parameter after flag -%c\n", c);
-            return 1; 
-          }
-
-          if (c=='O') {                               /* -O flag  */
-            if (!strcmp(aaa,"=")) {
-              choose_outname=1;
-            }
-            else {
-              getext (aaa,ext);
-              if (strcmp(ext,"ps") && strcmp(ext,"eps") && strcmp(ext,"")) {
-                printf ("Wrong extension for output file: %s\n", aaa);
+            
+            if (got_value && strchr("vskYB",c)) {      /* check num args */
+              ok=1;
+              for (j=0;j<strlen(aaa);j++)
+                if (!strchr("0123456789.",aaa[j])) ok=0;
+              if (!ok) {
+                printf ("+++ Invalid parameter <%s> for flag -%c\n",aaa,c);
                 return 1;
               }
-              strext (outf, aaa, "ps", 1);
-              choose_outname=0;
             }
-          }
+            
+            /* --- next ops require a value --- */
 
-          if (c=='a') {              
-	    sscanf (aaa, "%f", &alfa_c);
-            if ((alfa_c>1.05)||(alfa_c<-0.01)) {
-              printf ("++++ Bad parameter for flag -a: %s\n",aaa);
-              return 1;
+            if (!got_value) {                   /* check value was given */
+              printf ("+++ Missing parameter after flag -%c\n", c);
+              return 1; 
             }
-          }
-          
-          if (c=='B') {
-            sscanf(aaa,"%d",&bars_per_line); 
-            continue_lines=0;
-          }
-          if (c=='v') sscanf(aaa,"%d",&vb); 
-          if (c=='s') sscanf(aaa,"%f",&scalefac);
-          if (c=='d') staffsep = scan_u(aaa);
-          if (c=='w') swidth   = scan_u(aaa);
-          if (c=='m') lmargin  = scan_u(aaa);
-          if (c=='F') strcpy(styf,aaa);
-          if (c=='D') strcpy (styd,aaa);
-          if (c=='g') {
-            if      (abbrev(aaa,"shrink", 2)) gmode=G_SHRINK;
-            else if (abbrev(aaa,"stretch",2)) gmode=G_STRETCH;
-            else if (abbrev(aaa,"space",  2)) gmode=G_SPACE;
-            else if (abbrev(aaa,"fill",   2)) gmode=G_FILL;
-            else {
-              printf ("++++ Bad parameter for flag -g: %s\n",aaa);
-              return 1;
+
+            if (c=='k') {
+              sscanf(aaa,"%d",&barnums); 
+              break;
             }
+            
+            if (c=='V') {                               /* -V flag */
+              ok=1;
+              strcpy (vcselstr, aaa);
+            }
+            
+            if (c=='X') {                               /* -X flag */
+              ok=1;
+              if (aaa[0]==',') {
+                sscanf(aaa,",%f",&strict2);
+                if (strict2<-0.001 || strict2>1.001) ok=0;
+              }            
+              else if (strchr(aaa,',')) {
+                sscanf (aaa,"%f,%f",&strict1,&strict2);
+                if (strict1<-0.001 || strict1>1.001) ok=0;
+                if (strict2<-0.001 || strict2>1.001) ok=0;
+              }
+              else {
+                sscanf(aaa,"%f",&strict1);
+                if (strict1<-0.001 || strict1>1.001) ok=0;
+                strict2=strict1;
+              }
+              if (!ok) {
+                printf ("+++ Invalid parameter <%s> for flag -%c\n",aaa,c);
+                return 1;
+              }
+            }
+            
+            if (c=='O') {                               /* -O flag  */
+              if (!strcmp(aaa,"=")) {
+                choose_outname=1;
+              }
+              else {
+                getext (aaa,ext);
+                if (strcmp(ext,"ps") && strcmp(ext,"eps") && strcmp(ext,"")) {
+                  printf ("Wrong extension for output file: %s\n", aaa);
+                  return 1;
+                }
+                strext (outf, aaa, "ps", 1);
+                choose_outname=0;
+              }
+            }
+            
+            if (c=='a') {              
+              sscanf (aaa, "%f", &alfa_c);
+              if ((alfa_c>1.05)||(alfa_c<-0.01)) {
+                printf ("+++ Bad parameter for flag -a: %s\n",aaa);
+                return 1;
+              }
+            }
+            
+            if (c=='B') {
+              sscanf(aaa,"%d",&bars_per_line); 
+              continue_lines=0;
+            }
+            if (c=='t') strcpy(transpose,aaa);
+            if (c=='v') sscanf(aaa,"%d",&vb); 
+            if (c=='s') sscanf(aaa,"%f",&scalefac);
+            if (c=='d') {
+              if (aaa[0]=='+' || aaa[0]=='-') dstaffsep = scan_u(aaa);
+              else staffsep = scan_u(aaa);
+            }
+            if (c=='w') swidth   = scan_u(aaa);
+            if (c=='m') lmargin  = scan_u(aaa);
+            if (c=='F') strcpy(styf,aaa);
+            if (c=='D') strcpy (styd,aaa);
+            if (c=='g') {
+              if      (abbrev(aaa,"shrink", 2)) gmode=G_SHRINK;
+              else if (abbrev(aaa,"stretch",2)) gmode=G_STRETCH;
+              else if (abbrev(aaa,"space",  2)) gmode=G_SPACE;
+              else if (abbrev(aaa,"fill",   2)) gmode=G_FILL;
+              else {
+                printf ("+++ Bad parameter for flag -g: %s\n",aaa);
+                return 1;
+              }
+            }
+            break; 
           }
-          break; 
+          else {
+            printf ("+++ Unknown flag: -%c\n", c);
+            return 1;
+          }
+          m++;
         }
-        else {
-          printf ("++++ Unknown flag: -%c\n", c);
-          return 1;
-        }
-        m++;
       }
     }
 
@@ -365,7 +432,7 @@ char *av[];
         
         if (!sel_arg) {             /* this arg is a file name */
           if (ninf>=MAXINF) {
-            printf ("++++ Too many input files, max is %d\n", MAXINF);
+            printf ("+++ Too many input files, max is %d\n", MAXINF);
             return 1;
           }
           strcpy (in_file[ninf], av[i]); 
@@ -384,6 +451,47 @@ char *av[];
   return 0;
 }
 
+/* ----- alloc_structs ----- */
+/* Thanks to Henrik Norbeck for this */
+void alloc_structs ()
+{
+  int j;
+
+  sym = (struct SYMBOL *)calloc(maxSyms, sizeof(struct SYMBOL));
+  if (sym==NULL) rx("Out of memory","");
+
+  symv = (struct SYMBOL **)calloc(maxVc, sizeof(struct SYMBOL *));
+  if (symv==NULL) rx("Out of memory","");
+
+  for (j=0;j<maxVc;j++) {
+    symv[j] = (struct SYMBOL *)calloc(maxSyms, sizeof(struct SYMBOL));
+    if (symv[j]==NULL) rx("Out of memory","");
+  }
+  
+  xp = (struct XPOS *)calloc(maxSyms+1, sizeof(struct XPOS));
+  if (xp==NULL) rx("Out of memory","");
+  
+  for (j=0;j<maxSyms+1;j++) {
+    xp[j].p = (int *)calloc(maxVc, sizeof(int));
+    if (xp[j].p==NULL) rx("Out of memory","");
+  }
+
+  voice=(struct VCESPEC *)calloc(maxVc, sizeof(struct VCESPEC));
+  if (voice==NULL) rx("Out of memory","");
+
+  sym_st = (struct SYMBOL **)calloc(maxVc, sizeof(struct SYMBOL *));
+  if (sym_st==NULL) rx("Out of memory","");
+  
+  for (j=0;j<maxVc;j++) {
+    sym_st[j] = (struct SYMBOL *)calloc(MAXSYMST, sizeof(struct SYMBOL));
+    if (sym_st[j]==NULL) rx("Out of memory","");
+  }
+  
+  nsym_st = (int *)calloc(maxVc, sizeof(int));
+  if (nsym_st==NULL) rx("Out of memory","");
+
+}
+
 
 /* ----- set_page_format ----- */
 int set_page_format ()
@@ -397,13 +505,13 @@ int set_page_format ()
   else 
     set_standard_format (&cfmt);
 
-  i=read_fmt_file ("fonts.fmt", styd, &cfmt, 0);
+  i=read_fmt_file ("fonts.fmt", styd, &cfmt);
   j=0;
   if (strlen(styf)>0) {
     strext(styf,styf,"fmt",1);
-    j=read_fmt_file (styf, styd, &cfmt, 1);
+    j=read_fmt_file (styf, styd, &cfmt);
     if (j==0)  {
-      printf ("\n++++ Cannot open file: %s\n", styf);
+      printf ("\n+++ Cannot open file: %s\n", styf);
       return 0;
     }
     strcpy(cfmt.name, styf);
@@ -555,11 +663,11 @@ int add_bracket;
     if (!strcmp(font.name,fontnames[i])) fnum=i; 
   }
   if (fnum<0) {
-    printf ("\n++++ Font \"%s\" not predefined; using first in list\n",
+    printf ("\n+++ Font \"%s\" not predefined; using first in list\n",
             font.name); 
     fnum=0;
   }
-  PUT2("%.1f F%d ", font.size, fnum)
+  PUT3("%.1f %d F%d ", font.size, font.box, fnum)
   if (add_bracket) PUT0("(")
 }
 
@@ -574,7 +682,7 @@ struct FONTSPEC font;
   for (i=0;i<nfontnames;i++) {
     if (!strcmp(font.name,fontnames[i])) fnum=i; 
   }
-  sprintf (str,"%.1f F%d ", font.size, fnum);
+  sprintf (str,"%.1f %d F%d ", font.size, font.box, fnum);
 }
 
 
@@ -622,7 +730,7 @@ void close_output_file ()
   close_ps (fout);
   fclose (fout);
   if (tunenum==0) 
-    printf ("++++ Warning: no tunes written to output file\n");
+    printf ("+++ Warning: no tunes written to output file\n");
   m=get_file_size (outfnam);
   printf ("Output written on %s (%d page%s, %d title%s, %d byte%s)\n", 
           outfnam, 
@@ -687,11 +795,11 @@ int add_final_nl;
     *a='\0';
     if (strlen(word)>MAXWLEN) {
       word[MAXWLEN-1]='\0';
-      printf ("++++ Insanely long word truncated to %d chars: %s\n", 
+      printf ("+++ Insanely long word truncated to %d chars: %s\n", 
               MAXWLEN-1,word);
     }
     if (nt>=MAXNTEXT) {
-      printf ("\n++++ Line %d: %s\n", linenum,ln);
+      printf ("\n+++ Line %d: %s\n", linenum,ln);
       rx("Text overflow; increase MAXNTEXT and recompile.","");
     }
     if (strlen(word)>0) {
@@ -754,7 +862,7 @@ int  job;
         tex_str(txt[i],str,&wwidth);
         PUT1("%s ",str);
       }
-      PUT0(") show\n")
+      PUT0(") rshow\n")
     }
 
     else if (job==OBEYCENTER) {
@@ -863,7 +971,7 @@ FILE *fp;
         if (strlen(p)>0) {
           PUT0("50 0 M (")
           put_str (p);
-          PUT0(") show\n")
+          PUT0(") rshow\n")
         }
       }
     }
@@ -965,19 +1073,21 @@ FILE *fp;
 {
   char t[201];
 
-  bskip (cfmt.titlespace);
-  set_font (fp, cfmt.titlefont, 0);
-
   if      (numtitle==1) strcpy (t,info.title);
   else if (numtitle==2) strcpy (t,info.title2);
-  else if (numtitle==2) strcpy (t,info.title3);
+  else if (numtitle==3) strcpy (t,info.title3);
+  if (vb>15) printf ("write inside title <%s>\n", t);
+  if (strlen(t)==0) return;
+
+  bskip (cfmt.subtitlefont.size+0.2*CM);
+  set_font (fp, cfmt.subtitlefont, 0);
 
   if (cfmt.titlecaps) cap_str(t);
   PUT0(" (")
   put_str (t);
-  if (cfmt.titleleft) PUT0(") 0 0 M show\n")
+  if (cfmt.titleleft) PUT0(") 0 0 M rshow\n")
   else PUT1(") %.1f 0 M cshow\n", cfmt.staffwidth/2)  
-  bskip (cfmt.musicspace); 
+  bskip (cfmt.musicspace+0.2*CM);
 
 }  
 
@@ -992,11 +1102,133 @@ FILE *fp;
 }
 
 
+/* ----- write_tempo ----- */
+void write_tempo(fp,tempo,meter)
+FILE *fp;
+char tempo[];
+struct METERSTR meter;
+{
+  char *r, *q;
+  char text[STRL];
+  int top,bot,value,len,i,err,fac,dig,div;
+  struct SYMBOL s;
+  float stem,dotx,doty,sc,dx;
+
+  if (vb>15) printf ("write tempo <%s>\n", info.tempo);
+  r=tempo;
+  set_font (fp,cfmt.tempofont,0);
+  PUT0(" 18 0 M\n")
+
+  for (;;) {
+
+    while (*r==' ') r++;                    /* skip blanks */
+    if (*r=='\0') break;
+
+    if (*r=='"') {                          /* write string */
+      r++;
+      q=text;
+      while (*r!='"' && *r!='\0') { *q=*r; r++; q++; }
+      if (*r=='"') r++;
+      *q='\0';
+      if (strlen(text)>0) {     
+        PUT0("6 0 rmoveto (")
+        put_str (text);
+        PUT0(") rshow 12 0 \n")
+      }
+    }
+
+    else {                                  /* write tempo denotation */
+      q=text;
+      while (*r!=' ' && *r!='\0') { *q=*r; r++; q++; }
+      *q='\0';
+      
+      q=text;
+      len=QUARTER;
+      value=0;
+      err=0;
+      if (strchr(q,'=')) {
+        if (*q=='C' || *q=='c') {
+          q++;
+          len=meter.dlen;  
+          div=0;
+          if (*q=='/') { div=1; q++; }
+          fac=0;
+          while (isdig(*q)) { dig=*q-'0'; fac=10*fac+dig; q++; }
+
+          if (div) {
+            if (fac==0) fac=2;
+            if (len%fac) printf ("Bad length divisor in tempo: %s", text);
+            len=len/fac;
+          }
+          else  
+            if (fac>0) len=len*fac;
+          if (*q!='=') err=1;
+          q++;
+          if (!isdig(*q)) err=1;
+          sscanf(q,"%d", &value);
+        }
+        else if (isdig(*q)) {
+          sscanf(q,"%d/%d=%d", &top,&bot,&value);
+          len=(BASE*top)/bot;
+        }
+        else err=1;
+      } 
+      else {
+        if (isdig(*q))
+          sscanf(q,"%d", &value);
+        else err=1;
+      }
+      
+      if (err)                              /* draw note=value */
+        printf("\n+++ invalid tempo specifier: %s\n", text);
+      else {
+        s.len=len;
+        identify_note (&s,r);
+        sc=0.55*cfmt.tempofont.size/10.0;
+        PUT2("gsave %.2f %.2f scale 15 3 rmoveto currentpoint\n", sc,sc)
+        if (s.head==H_OVAL)  PUT0("HD")
+        if (s.head==H_EMPTY) PUT0("Hd")
+        if (s.head==H_FULL)  PUT0("hd")
+        dx=4.0;
+        if (s.dots) {      
+          dotx=8; doty=0;
+          if (s.flags>0) dotx=dotx+4;
+          if (s.head==H_EMPTY) dotx=dotx+1;
+          if (s.head==H_OVAL)  dotx=dotx+2;
+          for (i=0;i<s.dots;i++) {
+            PUT2(" %.1f %.1f dt", dotx, doty)
+            dx=dotx;
+            dotx=dotx+3.5;
+          }
+        }
+        stem=16.0;
+        if (s.flags>1) stem=stem+3*(s.flags-1);
+        if (s.len<WHOLE) PUT1(" %.1f su",stem) 
+        if (s.flags>0) PUT2(" %.1f f%du",stem,s.flags)
+        if ((s.flags>0) && (dx<6.0)) dx=6.0;
+        dx=(dx+18)*sc;
+        PUT2(" grestore %.2f 0 rmoveto ( = %d) rshow\n", dx,value)
+      }
+    }
+  }
+}
+
+
+/* ----- write_inside_tempo  ----- */
+void write_inside_tempo (fp)
+FILE *fp;
+{
+
+  bskip (cfmt.partsfont.size);
+  write_tempo(fp,info.tempo,voice[ivc].meter);
+  bskip (0.1*CM);
+}
+
 /* ----- write_heading  ----- */
 void write_heading (fp)
 FILE *fp;
 {
-  float lwidth;
+  float lwidth,down1,down2;
   int i,ncl;
   char t[201];
 
@@ -1009,7 +1241,7 @@ FILE *fp;
   strcpy (t,info.title);
   if (cfmt.titlecaps) cap_str(t);
   put_str (t);
-  if (cfmt.titleleft) PUT0(") 0 0 M show\n")
+  if (cfmt.titleleft) PUT0(") 0 0 M rshow\n")
   else PUT1(") %.1f 0 M cshow\n", lwidth/2)  
 
   /* write second title */
@@ -1019,7 +1251,7 @@ FILE *fp;
     strcpy (t,info.title2);
     if (cfmt.titlecaps) cap_str(t);
     put_str (t);
-    if (cfmt.titleleft) PUT0(") 0 0 M show\n")
+    if (cfmt.titleleft) PUT0(") 0 0 M rshow\n")
     else PUT1(") %.1f 0 M cshow\n", lwidth/2)  
   }
 
@@ -1030,10 +1262,10 @@ FILE *fp;
     strcpy (t,info.title3);
     if (cfmt.titlecaps) cap_str(t);
     put_str (t);
-    if (cfmt.titleleft) PUT0(") 0 0 M show\n")
+    if (cfmt.titleleft) PUT0(") 0 0 M rshow\n")
     else PUT1(") %.1f 0 M cshow\n", lwidth/2)  
   }
-  
+
   /* write composer, origin */
   if ((info.ncomp>0) || (strlen(info.orig)>0)) {
     set_font (fp,cfmt.composerfont,0);
@@ -1051,33 +1283,44 @@ FILE *fp;
       }
       PUT0 (") lshow\n");
     }
-/* next part puts origin in separate line */
-/*    if (strlen(info.orig)>0) {
-      bskip(cfmt.composerfont.size);
-      PUT1("%.1f 0 M (", lwidth)
-      put_str (" (");
-      put_str (info.orig);
-      put_str (")");
-      PUT0(") lshow\n") 
-    }  */
+    down1=cfmt.composerspace+cfmt.musicspace+ncl*cfmt.composerfont.size;
   }
   else {
     bskip(cfmt.composerfont.size+cfmt.composerspace);
+    down1=cfmt.composerspace+cfmt.musicspace+cfmt.composerfont.size;
   }
+  bskip (cfmt.musicspace);
 
-  if (strlen(info.parts)>0) {
-    bskip (cfmt.musicspace-cfmt.partsspace);
-    set_font (fp,cfmt.partsfont,0);
-    PUT0("0 0 M (")
-    put_str (info.parts);
-    PUT0(") show\n")
-    bskip (cfmt.partsspace);
-  }
-  else {
-    bskip(cfmt.musicspace);
-  }
+  /* decide whether we need extra shift for parts and tempo */
+  down2=cfmt.composerspace+cfmt.musicspace;
+  if (strlen(info.parts)>0) down2=down2+cfmt.partsspace+cfmt.partsfont.size;
+  if (strlen(info.tempo)>0) down2=down2+cfmt.partsspace+cfmt.partsfont.size;
+  if (down2>down1) bskip (down2-down1);
 
-/*  buffer_eob (fp); */
+  /* write tempo and parts */
+  if (strlen(info.parts)>0 || strlen(info.tempo)>0) {
+    
+    if (strlen(info.tempo)>0) {
+      bskip (-0.2*CM);
+      PUT1 (" %.2f 0 T ", cfmt.indent*cfmt.scale)
+      write_tempo(fp, info.tempo, default_meter);
+      PUT1 (" %.2f 0 T ", -cfmt.indent*cfmt.scale)
+      bskip (-cfmt.tempofont.size);
+    }
+    
+    if (strlen(info.parts)>0) {
+      bskip (-cfmt.partsspace);
+      set_font (fp,cfmt.partsfont,0);
+      PUT0("0 0 M (")
+      put_str (info.parts);
+      PUT0(") rshow\n")
+      bskip (cfmt.partsspace);
+    }
+
+    if (strlen(info.tempo)>0) bskip (cfmt.tempofont.size+0.3*CM);
+    
+  }
+  
 
 }
 
@@ -1090,7 +1333,8 @@ FILE *fp;
     set_font (fp, cfmt.partsfont,0);
     PUT0("0 0 M (")
     put_str (info.parts);
-    PUT0(") show\n")
+    PUT0(") rshow\n")
     bskip(cfmt.partsspace);
   }
 }   
+
